@@ -1,3 +1,7 @@
+dir = File.expand_path(File.join(File.dirname(__FILE__), '..', 'lib'))
+require File.join(dir, 'httparty')
+require 'pp'
+
 class ParseAtom
   include HTTParty
 
