@@ -1,3 +1,4 @@
+require 'json'
 require 'spec_helper'
 
 RSpec.describe HTTParty::Logger::LogstashFormatter do
@@ -6,7 +7,8 @@ RSpec.describe HTTParty::Logger::LogstashFormatter do
   let(:path) { 'http://my.domain.com/my_path' }
   let(:logger_double) { double('Logger') }
   let(:request_double) { double('Request', http_method: Net::HTTP::Get, path: "#{path}") }
-  let(:request_time) { Time.new.strftime("%Y-%m-%d %H:%M:%S %z") }
+  let(:frozen_time) { Time.new(2024, 1, 1, 12, 0, 0, "+00:00") }
+  let(:request_time) { frozen_time.strftime("%Y-%m-%d %H:%M:%S %z") }
   let(:log_message) do
     {
       '@timestamp' => request_time,
@@ -24,6 +26,7 @@ RSpec.describe HTTParty::Logger::LogstashFormatter do
   subject { described_class.new(logger_double, severity) }
 
   before do
+    allow(Time).to receive(:now).and_return(frozen_time)
     expect(logger_double).to receive(:info).with(log_message)
   end
 
