@@ -245,6 +245,58 @@ RSpec.describe HTTParty::Request do
       expect(request.instance_variable_get(:@raw_request).body_stream).to eq(stream)
     end
 
+    context 'with file upload' do
+      let(:file) { File.open('spec/fixtures/tiny.gif', 'rb') }
+      after { file.close }
+
+      it 'should use non-streaming body by default for backwards compatibility' do
+        request = HTTParty::Request.new(Net::HTTP::Post, 'http://api.foo.com/v1', body: { avatar: file })
+        request.send(:setup_raw_request)
+        raw_request = request.instance_variable_get(:@raw_request)
+
+        expect(raw_request.body_stream).to be_nil
+        expect(raw_request.body).not_to be_nil
+        expect(raw_request.body).to be_a(String)
+      end
+
+      it 'should use body_stream when stream_body is true' do
+        request = HTTParty::Request.new(Net::HTTP::Post, 'http://api.foo.com/v1', body: { avatar: file }, stream_body: true)
+        request.send(:setup_raw_request)
+        raw_request = request.instance_variable_get(:@raw_request)
+
+        expect(raw_request.body_stream).to be_a(HTTParty::Request::StreamingMultipartBody)
+        expect(raw_request.body).to be_nil
+        expect(raw_request['Content-Length']).not_to be_nil
+      end
+
+      it 'should set correct Content-Length header from stream size' do
+        request = HTTParty::Request.new(Net::HTTP::Post, 'http://api.foo.com/v1', body: { avatar: file }, stream_body: true)
+        request.send(:setup_raw_request)
+        raw_request = request.instance_variable_get(:@raw_request)
+
+        stream = raw_request.body_stream
+        expect(raw_request['Content-Length'].to_i).to eq(stream.size)
+      end
+
+      it 'streaming body produces same content as non-streaming' do
+        allow(HTTParty::Request::MultipartBoundary).to receive(:generate).and_return('test-boundary-123')
+
+        # Get streaming content
+        request1 = HTTParty::Request.new(Net::HTTP::Post, 'http://api.foo.com/v1', body: { avatar: file, first_name: "John" }, stream_body: true)
+        request1.send(:setup_raw_request)
+        streaming_content = request1.instance_variable_get(:@raw_request).body_stream.read
+
+        file.rewind
+
+        # Get non-streaming content
+        request2 = HTTParty::Request.new(Net::HTTP::Post, 'http://api.foo.com/v1', body: { avatar: file, first_name: "John" })
+        request2.send(:setup_raw_request)
+        non_streaming_content = request2.instance_variable_get(:@raw_request).body
+
+        expect(streaming_content).to eq(non_streaming_content)
+      end
+    end
+
     it 'should normalize base uri when specified as request option' do
       stub_request(:get, 'http://foo.com/resource').to_return(body: 'Bar')
       response = HTTParty.get('/resource', {
@@ -332,6 +384,74 @@ RSpec.describe HTTParty::Request do
         end
       end
     end
+
+    context "URI safety validation" do
+      context "when base_uri is configured" do
+        it "raises UnsafeURIError when path is an absolute URL with different host" do
+          request = HTTParty::Request.new(
+            Net::HTTP::Get,
+            'http://evil.com/steal-data',
+            base_uri: 'http://trusted.com'
+          )
+          expect { request.uri }.to raise_error(
+            HTTParty::UnsafeURIError,
+            /has host 'evil.com' but the configured base_uri .* has host 'trusted.com'/
+          )
+        end
+
+        it "allows requests when path host matches base_uri host" do
+          request = HTTParty::Request.new(
+            Net::HTTP::Get,
+            'http://trusted.com/api/data',
+            base_uri: 'http://trusted.com'
+          )
+          expect { request.uri }.not_to raise_error
+          expect(request.uri.host).to eq('trusted.com')
+        end
+
+        it "allows relative paths" do
+          request = HTTParty::Request.new(
+            Net::HTTP::Get,
+            '/api/data',
+            base_uri: 'http://trusted.com'
+          )
+          expect { request.uri }.not_to raise_error
+          expect(request.uri.to_s).to eq('http://trusted.com/api/data')
+        end
+
+        it "raises UnsafeURIError for network-relative URLs with different host" do
+          @request.last_uri = URI.parse("https://trusted.com")
+          @request.path = URI.parse("//evil.com/steal")
+          @request.redirect = true
+          @request.options[:base_uri] = 'http://trusted.com'
+
+          # During redirects, URI safety is not checked to allow legitimate redirects
+          expect { @request.uri }.not_to raise_error
+        end
+
+        it "can be bypassed with skip_uri_validation option" do
+          request = HTTParty::Request.new(
+            Net::HTTP::Get,
+            'http://other.com/api/data',
+            base_uri: 'http://trusted.com',
+            skip_uri_validation: true
+          )
+          expect { request.uri }.not_to raise_error
+          expect(request.uri.host).to eq('other.com')
+        end
+      end
+
+      context "when base_uri is not configured" do
+        it "allows absolute URLs" do
+          request = HTTParty::Request.new(
+            Net::HTTP::Get,
+            'http://any-server.com/api/data'
+          )
+          expect { request.uri }.not_to raise_error
+          expect(request.uri.host).to eq('any-server.com')
+        end
+      end
+    end
   end
 
   describe "#setup_raw_request" do
@@ -378,6 +498,29 @@ RSpec.describe HTTParty::Request do
         end
       end
     end
+
+    context "when body is a Hash" do
+      subject(:headers) do
+        @request.send(:setup_raw_request)
+        headers = @request.instance_variable_get(:@raw_request).each_header.to_a
+        Hash[*headers.flatten]
+      end
+
+      it "sets header Content-Type: application/x-www-form-urlencoded" do
+        @request.options[:body] = { foo: 'bar' }
+
+        expect(headers['content-type']).to eq('application/x-www-form-urlencoded')
+      end
+
+      context "and header Content-Type is provided" do
+        it "does not overwrite the provided Content-Type" do
+          @request.options[:body] = { foo: 'bar' }
+          @request.options[:headers] = { 'Content-Type' => 'application/json' }
+
+          expect(headers['content-type']).to eq('application/json')
+        end
+      end
+    end
   end
 
   describe 'http' do
@@ -928,6 +1071,14 @@ RSpec.describe HTTParty::Request do
           expect(@request.http_method).to eq(Net::HTTP::Delete)
         end
 
+        it 'should clear the body before resulting GET requests' do
+          @request.http_method = Net::HTTP::Post
+          @request.options[:body] = { text: 'something' }
+          expect(@request.perform.parsed_response).to eq({"hash" => {"foo" => "bar"}})
+          expect(@request.http_method).to eq(Net::HTTP::Get)
+          expect(@request.options[:body]).to be_nil
+        end
+
         it 'should log the redirection' do
           logger_double = double
           expect(logger_double).to receive(:info).twice
@@ -1361,6 +1512,20 @@ RSpec.describe HTTParty::Request do
         @request.send(:setup_raw_request)
         expect(@request.instance_variable_get(:@raw_request)['authorization']).to eq(@authorization)
       end
+
+      context 'when uri path is a relative path' do
+        before do
+          @request.path = '/v1'
+          @request.options[:base_uri] = 'http://api.foo.com'
+        end
+
+        it "should send Authorization header when redirecting to the same host" do
+          @redirect['location'] = 'http://api.foo.com/v2'
+          @request.perform
+          @request.send(:setup_raw_request)
+          expect(@request.instance_variable_get(:@raw_request)['authorization']).to eq(@authorization)
+        end
+      end
     end
   end
 
